--- old/usr/src/uts/common/os/driver_lyr.c	Mon Nov 26 00:31:49 2012
+++ new/usr/src/uts/common/os/driver_lyr.c	Mon Nov 26 00:31:49 2012
@@ -144,6 +144,9 @@
 	{ NULL}			/* must terminate list */
 };
 
+static ldi_ev_callback_impl_t *walker_next = NULL;
+static ldi_ev_callback_impl_t *walker_prev = NULL;
+
 void
 ldi_init(void)
 {
@@ -3329,8 +3332,10 @@
 
 	ret = LDI_EV_NONE;
 	ldi_ev_lock();
+	VERIFY(walker_next == NULL);
 	listp = &ldi_ev_callback_list.le_head;
-	for (lecp = list_head(listp); lecp; lecp = list_next(listp, lecp)) {
+	for (lecp = list_head(listp); lecp; lecp = walker_next) {
+		walker_next = list_next(listp, lecp);
 
 		/* Check if matching device */
 		if (!ldi_ev_device_match(lecp, dip, dev, spec_type))
@@ -3386,7 +3391,9 @@
 	 * Undo notifies already sent
 	 */
 	lecp = list_prev(listp, lecp);
-	for (; lecp; lecp = list_prev(listp, lecp)) {
+	VERIFY(walker_prev == NULL);
+	for (; lecp; lecp = walker_prev) {
+		walker_prev = list_prev(listp, lecp);
 
 		/*
 		 * Check if matching device
@@ -3437,6 +3444,8 @@
 	}
 
 out:
+	walker_next = NULL;
+	walker_prev = NULL;
 	ldi_ev_unlock();
 
 	if (ret == LDI_EV_NONE) {
@@ -3552,8 +3561,10 @@
 	    " event=%s", (void *)dip, ldi_result, event));
 
 	ldi_ev_lock();
+	VERIFY(walker_next == NULL);
 	listp = &ldi_ev_callback_list.le_head;
-	for (lecp = list_head(listp); lecp; lecp = list_next(listp, lecp)) {
+	for (lecp = list_head(listp); lecp; lecp = walker_next) {
+		walker_next = list_next(listp, lecp);
 
 		if (lecp->lec_finalize == NULL) {
 			LDI_EVDBG((CE_NOTE, "ldi_invoke_finalize(): No "
@@ -3604,6 +3615,7 @@
 			lecp->lec_finalize = NULL;
 		}
 	}
+	walker_next = NULL;
 	ldi_ev_unlock();
 
 	if (found)
@@ -3685,6 +3697,13 @@
 		next = list_next(listp, lecp);
 		if (lecp->lec_id == id) {
 			ASSERT(found == NULL);
+
+			/* If there is a walk in progress, move it along... */
+			if (walker_next == lecp)
+				walker_next = next;
+			if (walker_prev == lecp)
+				walker_prev = list_prev(listp, walker_prev);
+
 			list_remove(listp, lecp);
 			found = lecp;
 		}
